#!/usr/bin/perl

# Version: 1.2.
# Author: Leonid Mamtchenkov
# Date: Fri 11 May 2001 05:26:56 PM EEST
# TODO: 
#	- Add totals/averages.
#	- Clean up.
#	- Optimize.
#	- Add html support.

use POSIX qw(strftime);

$workdir="/root/bin/squid";
$tempfile="$workdir/proxy-report.tmp";
$resultfile="$workdir/proxy-report.txt";
$mylogfile="$workdir/proxy-report.log";
$ip2find="127.0.0.1";
$ignoreurl="";
$logdir=shift;
$logfilenamepattern="access";
###############################################################################
&line2log("-"x50);

if (-d $logdir) {
    $logdir=~s/\/$//;
    &line2log("Searching for '*$logfilenamepattern*'-files in directory $logdir");
    @filenames=`ls -1 $logdir | grep $logfilenamepattern`;
}
else {
    &line2log("ERROR: $logdir does not seem to be a directory");
    exit(1);
}

foreach $foundlogfile (@filenames) {
    chomp($foundlogfile);
    &line2log("Found $foundlogfile");
    ($logfile,$compressed)=&decompresslogfile("$logdir/$foundlogfile");
    &processlogfile($logfile);
    if ($compressed eq 1) { 
	&line2log("Compressing $logfile"); 
	system("gzip $logfile");
    }
}
&createproxyreport;
unlink ($tempfile);
###############################################################################
sub decompresslogfile{
    $file2check=shift;
    if ($file2check=~/\.gz/) {
	&line2log("Decompressing $file2check");
	system ("gzip -d $file2check");
	$file2check=~s/\.gz//;
	$compressed=1;
    }
    else { 
	&line2log("File $file2check is not compressed");
	$compressed=0;
    }
    return $file2check,$compressed;
}

sub processlogfile {
    $inputfile=shift;
    &line2log("Processing $inputfile");
    open (IN, $inputfile) or die "ERROR: Could not read from input file ($!)\n";
    open (OUT, ">>$tempfile") or die "ERROR: Could not read from input file ($!) \n";
    while (chomp($line=<IN>)) {
	@fields = split (/\s+/, $line);
        $reqtime = $fields[0]; # Time or the request
        $reqip  = $fields[2];  # IP address from which request came.
        $requrl  = $fields[6]; # URL requested

        if ($requrl =~ /:\/\//) {
	    ($proto, $server) = split (/:\/\//, $requrl); 
	}
        else { 
	    $proto="http"; $server=$requrl; 
	}
        ($fqdn, @path) = split (/\//, $server);
        $requrl="$proto://$fqdn";

        if (($reqip=~/$ip2find/) && 
        (!($requrl=~/$ignoreurl/))) {
#	&& ($requrl ne $oldurl)) {
	
#	    $currentdate=strftime "%d %b %Y (%a)", localtime($reqtime);
	    $currentdate=strftime "%Y-%m-%d (%a)", localtime($reqtime);
	    print OUT "$currentdate\t";
	    $olddate=$currentdate;
	    print OUT "$requrl\n";
	    $oldurl=$requrl;
	}

    }
    close (OUT);
    close (IN);
}

sub createproxyreport {
    &line2log("Generating report to file $resultfile");
    open (IN, $tempfile);
    while (chomp($inline=<IN>)) {
#	($date,$url)=split(/\t/, $inline);
	$counter{$inline}++;
    }
    close (IN);
    open (OUT, ">$resultfile");
    
    $reportdate=strftime "%d %b %Y (%a)", localtime(time);
    $reporttime=strftime "%H:%M:%S", localtime(time);
    print OUT "Proxy server statistics report\n";
    print OUT "------------------------------\n";
    print OUT "Date: $reportdate\t$reporttime\n";
    print OUT "Monitored IP address: $ip2find\n\n";
    
    foreach $key (reverse sort keys %counter) {
	($date,$url)=split(/\t/,$key);
	if ($date ne $olddate) {
	    @stats = reverse sort {$a <=> $b} @stats;
	    foreach $statline (@stats) {
		print OUT $statline;
	    }
	    undef(@stats);
	
	    print OUT "\n$date\n";
	    print OUT "# Requests\tURL\n";
	    $olddate=$date;
	}
	push @stats, "$counter{$key}\t\t$url\n";
    }
    @stats = reverse sort {$a <=> $b} @stats;
    foreach $statline (@stats) {
	print OUT $statline;
    }
    undef(@stats);
    close (OUT);
}

sub line2log {
    $logline=shift;
    $now=strftime "%Y-%b-%d %H:%M:%S", localtime(time);
    open (LOG, ">>$mylogfile");
    print LOG "$now\t$logline\n";
    close (LOG);
}
